//////////////////////////////////////////////////////////////////////////////
// Praktikum Informatik 1
// Versuch 11: Anwendung von Filterfunktionen
//
// Datei:  EdgeDetection.cpp
// Inhalt: Klasse EdgeDetector zur Detektierung von Kanten in Farb-/Graubildern
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
///////////////////////////////// INCLUDES ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <math.h>
#include <QtGui/QImage>
#include <QtGui/QColor>
#include "EdgeDetector.h"
using namespace std;

//////////////////////////////////////////////////////////////////////////////
///////////////////////// Funktions-Implementierung //////////////////////////
//////////////////////////////////////////////////////////////////////////////

EdgeDetector::EdgeDetector()
{
	// Horizontaler Sobelfilter
	filter = Matrix3x3(
										 1, 0, -1,
										 2, 0, -2,
										 1, 0, -1);
	scalFactor = 1;
}


void EdgeDetector::setFilter(Matrix3x3 m)
{
	filter = m;
}

void EdgeDetector::transposeMatrix()
{
	filter = filter.transpose();
}

void EdgeDetector::applyFilter(QImage * in, QImage * out)
{
	
	for (int i=1; i < in->width()-1; ++i)
	{
		for (int j=1; j < in->height() -1; ++j)
		{
			int grayVal=0;
			for (int mx=-1; mx <= 1; ++mx)
			{
				for (int my=-1; my <= 1; ++my)
				{	
					int tempVal = qGray(in->pixel(i+mx, j+my));
					grayVal += tempVal * filter.getMatrixWert(my+1, mx+1) * scalFactor;
				}
			}	
			grayVal = abs(grayVal);
			if (255 < grayVal)
			{
				grayVal = 255;
			}
			out->setPixel(i, j, QColor::fromRgb(grayVal, grayVal, grayVal, 0).rgb());
		}
	}
}

void EdgeDetector::applyFilterBothDirections(QImage * in, QImage * out, int doNorm)
{
	Matrix3x3 transFilter = filter.transpose();
	
	int tempFeld[2][in->width()][in->height()];
	int maxVal = 0;
	
	for (int i=1; i < in->width()-1; ++i)
	{
		for (int j=1; j < in->height() -1; ++j)
		{
			int grayValHor=0;
			int grayValVer=0;
			for (int mx=-1; mx <= 1; ++mx)
			{
				for (int my=-1; my <= 1; ++my)
				{	
					int tempVal = qGray(in->pixel(i+mx, j+my));
					grayValHor += tempVal * filter.getMatrixWert(my+1, mx+1) * scalFactor;
					grayValVer += tempVal * transFilter.getMatrixWert(my+1, mx+1) * scalFactor;
				}
			}	
			grayValHor = abs(grayValHor);
			if (255 < grayValHor)	grayValHor = 255;
			grayValVer = abs(grayValVer);
			if (255 < grayValVer) grayValVer = 255;
			
			tempFeld[0][i][j] = grayValHor;
			tempFeld[1][i][j] = grayValVer;
			
			int curVal = sqrt((double)(grayValHor*grayValHor + grayValVer*grayValVer));
			if (curVal > maxVal) maxVal = curVal;
		}
	}
	
	for (int i=1; i < in->width()-1; ++i)
	{
		for (int j=1; j < in->height() -1; ++j)
		{
			int sum = sqrt((double)(tempFeld[0][i][j]*tempFeld[0][i][j] + tempFeld[1][i][j]*tempFeld[1][i][j]));
			
			int scaledSum;
			if (doNorm)
			{
			 scaledSum = (double)255 / maxVal * sum;
			}
			else
			{
			  scaledSum = abs(sum);
				if (255 < scaledSum) scaledSum=255;
			}
			out->setPixel(i, j, QColor::fromRgb(scaledSum, scaledSum, scaledSum, 0).rgb());
		}
	}
}

void EdgeDetector::setScaleFactor(double newFactor)
{
	scalFactor = newFactor;
}

