#ifndef EXPRESSION_H
#define EXPRESSION_H

#include <iostream>
#include <string>
#include <sstream>

// Abstrakte Interface Klasse
class Expression
{
public:
	virtual ~Expression(){};
	virtual double evaluate() = 0;
	virtual std::string print() const = 0;
};


class Result : public Expression
{
public:
  Result(Expression * a)
  {
    _expression = a;
  }
  virtual double evaluate()
  {
    return _expression->evaluate();
  }
  virtual std::string print() const
  {
    return _expression->print();
  }
  virtual
  ~Result()
  {
    delete _expression;
    _expression = NULL;
  }

private:
  Expression* _expression;
};

class Const : public Expression
{
public:
  Const(double val)
  {
    _val = val;
  }
  virtual double evaluate()
  {
    return _val;
  }
  virtual std::string print() const
  {
    std::stringstream ss;
    ss << _val;
    return (ss.str());
  }
  virtual
  ~Const(){}

private:
  double _val;
};

class Add : public Expression
{
public:
  Add(Expression* a, Expression* b)
  {
    _left = a;
    _right = b;
  }
  virtual double evaluate()
  {
    return _left->evaluate() + _right->evaluate();
  }
  virtual std::string print() const
  {
    std::stringstream ss;
    ss << "(" << _left->print() << ")+(" << _right->print() << ")";
    return (ss.str());
  }
  virtual
  ~Add()
  {
    delete _left;
    delete _right;
  }

private:
  Expression* _left;
  Expression* _right;
};

class Sub : public Expression
{
public:
  Sub(Expression* a, Expression* b)
  {
    _left = a;
    _right = b;
  }
  virtual double evaluate()
  {
    return _left->evaluate() - _right->evaluate();
  }
  virtual std::string print() const
  {
    std::stringstream ss;
    ss << "(" << _left->print() << ")-(" << _right->print() << ")";
    return (ss.str());
  }
  virtual
  ~Sub()
  {
    delete _left;
    delete _right;
  }

private:
  Expression* _left;
  Expression* _right;
};

class Mul : public Expression
{
public:
  Mul(Expression* a, Expression* b)
  {
    _left = a;
    _right = b;
  }
  virtual double evaluate()
  {
    return _left->evaluate() * _right->evaluate();
  }
  virtual std::string print() const
  {
    std::stringstream ss;
    ss << "(" << _left->print() << ")*(" << _right->print() << ")";
    return (ss.str());
  }
  virtual
  ~Mul()
  {
    delete _left;
    delete _right;
  }

private:
  Expression* _left;
  Expression* _right;
};

class Div : public Expression
{
public:
  Div(Expression* a, Expression* b)
  {
    _left = a;
    _right = b;
  }
  virtual double evaluate()
  {
    return _left->evaluate() / _right->evaluate();
  }
  virtual std::string print() const
  {
    std::stringstream ss;
    ss << "(" << _left->print() << ")/(" << _right->print() << ")";
    return (ss.str());
  }
  virtual
  ~Div()
  {
    delete _left;
    delete _right;
  }
private:
  Expression* _left;
  Expression* _right;
};

#endif
