/*
 * Dieser Quellcode stellt die Basis des kompletten Programmes, das in dieser Teilaufgabe
 * entwickelt werden soll dar. Folgen sie zur Vervollstaendigung des Programmes an der entsprechend
 * gekennzeichneten Stelle ihren eigenen Quellcode ein. Diese Stelle ist wie folgt gekennzeichnet:
 * // Ihre Implentierung
 */

#include <cstdlib> // rand()
#include <iostream>
#include <ctime> // Fuer Initialisierung der random-Funktion
#include <vector>
#include <cmath>


#define RAND(x) (unsigned short)(rand() % x) // Makrofunktion - Gibt Zufallszahl zwischen 0 und x-1 zurueck
// Fuer solche Zwecke gibt es besser geeignete Funktionen, die aber auch groessere Implementierungen benoetigen
// und aus externen Quellen eingebunden oder selbst entwickelt werden muessen. Mit den Mitteln, die
// c++ zur Verfuegung stellt ist dies eine einfache Loesung


typedef unsigned short Vektorelement; // Die Elemente eines Vektors

using namespace std;

const unsigned short short_grenze (sizeof(short) * 8);

// Anzahl der Speicherworte
unsigned int iAnzahlSpeicherworte = 0;

vector<Vektorelement> vUEDE;

/*
 *  Dazu muss jede Bitstelle mit ihrem entsprechenden Hexadezimalaequivalent verglichen werden
 *  Bsp: Niedrigste Bitstelle & 1, zweite Bitstelle & 2, dritte Bitstelle & 4 usw.
 *  Wird als Ergebnis true zurueckgegeben, so muss die Zahl gesetzter Bits um 1 erhoeht werden
 */
unsigned int anzahlGesetzterBits(unsigned short bitmuster)
{
  unsigned int Einsenzahl = 0;

  for(unsigned int i = 0 ; i < short_grenze; i++)
  {
      if (bitmuster & ( 1 << i ))
      {
          Einsenzahl += 1;
      }
  }
  return Einsenzahl;
}


/*
 * Da der Vektor aus Short-Elementen (random kann nur ein signed short fuellen)
 * besteht und jedes davon n Bit enthaelt, besteht bei der Eingabe die Moeglichkeit,
 * Laengen der UEDE einzugeben, die kein Vielfaches von n darstellen. Daher muss
 * sichergestellt werden, dass auch diese "ungeraden" Werte erstellt werden koennen.
 * Es werden also zwei Variablen eingefuehrt, die ermoeglichen, die Laenge einer UEDE
 * korrekt zu generieren.
 */
unsigned int laengeDerUEDE(unsigned int codeLaenge)
{
  unsigned int laenge = 0;

  laenge = codeLaenge / short_grenze;

  if ((codeLaenge % short_grenze) > 0)
    laenge += 1;

  return laenge;
}


/*
 * Erzeugt zufaellige Bitfolgen vorgegebener Laenge, und fuegt
 * diese in einen Vektor ein
 */
void generiereUEDEs(unsigned int anzahl, unsigned int laenge)
{
  unsigned int rest = laenge % short_grenze;

  // Erzeuge vorgebene Anzahl UEDEs
  for (unsigned int i = 0; i < anzahl; ++i)
    {
      // Ausreichend Speicherworte fuer UEDEs
      for (unsigned int j = 0; j < iAnzahlSpeicherworte; ++j)
      {
          // laenge der UEDE kein vielfaches von sizeof(short)
          if ( (rest > 0) && ((j % laenge) == 0)){

              // hoechstwertigstes Element (mit fuehrenden Nullen) zuerst
              // erzeuge rest-stellige bitfolge
              vUEDE.push_back(RAND((1 << rest)));
          }
          else

              // 65535: hoechste 16-bit zahl
              vUEDE.push_back(RAND(65636));
      }
    }
}



/*
 * Liefert die Distanz zweier UEDE zurueck
 */
unsigned int distanz(vector<unsigned short> a, vector<unsigned short> b)
{
  int dist=0;

  for (int n=0; n < iAnzahlSpeicherworte; ++n)
  {
    Vektorelement xored = a[n] ^ b[n];
    dist += anzahlGesetzterBits(xored);
  }
  return dist;
}



/*
 * Liefert die UEDE von uebergebener Position pos
 */
vector<Vektorelement> getUEDE(unsigned int pos)
{
  vector<Vektorelement> tmp;

  /*
   * Die erste UEDE beginnt an Stelle 0, und endet an Stelle (0+iAnzahlSpeicherworte)
   * Die n-te UEDE beginnt an Stelle start=(n-1)*iAnzahlSpeicherworte und endet an Stelle (start+iAnzahlSpeicherworte)
   *
   * Achtung: Die uebergebenen Positionen beginnen bei Null, entsprechend der
   * Indexierung des Vektors: 1. Element an Stellle 0
   */
  for (unsigned int i=0; i < iAnzahlSpeicherworte; ++i)
    {
      tmp.push_back(vUEDE[pos*iAnzahlSpeicherworte + i]);
    }

  return tmp;
}

/*
 * Ausgabe einer UEDE
 */
void ausgabeUEDE(vector<Vektorelement> uede)
{
  // Alle Speicherworte der UEDE
  for (unsigned int j=0; j < uede.size(); ++j)
    {
      // Speicherwort bitweise ausgeben, MSB...LSB
      for (int k=short_grenze-1; k>=0; --k)
      {
          if (uede[j] & ( 1 << k ))
          {
              cout << "1";
          }
          else
          {
              cout << "0";
          }
      }
    }
  cout << endl;
}


/*
 * Berechnet die minimale Distanz ueber alle UDEs, gibt diese
 * mit dem UEDE-Paar aus
 */
void minimaleDistanz(unsigned int anzahlUEDE)
{
  vector<Vektorelement> a,b;

  // maximaler Wert fuer unsigned int
  unsigned int minimum = UINT_MAX;

  // Position der UEDEs mit minimaler Distanz
  unsigned int min_a, min_b;

  // Paarweise Vergleich aller UEDEs
  for (unsigned int i=0; i < anzahlUEDE; ++i)
    {
      for (unsigned j=i+1; j < anzahlUEDE; ++j)
      {
          a = getUEDE(i);
          b = getUEDE(j);

          // alle UEDE-Paare mit minimaler Distanz
          if (distanz(a,b) < minimum)
          {
              min_a = i;
              min_b = j;

              minimum = distanz(a,b);
          }
      }
    }
  cout << endl << "Die minimale Distanz lautet:\t" << minimum << endl;

  cout << "Die zugehoerigen UEDEs:" << endl;
  ausgabeUEDE(getUEDE(min_a));
  ausgabeUEDE(getUEDE(min_b));
}



int main(void)
{

    cout << "Versuch 11: Teilaufgabe 1 -> Ermitteln der minimalen Distanz" << endl;

    // random-Funktion ueber Sekundenanzahl initialisieren
    srand((unsigned) time(NULL));

    // Anzahl und Laenge der Datenuebertragungseinheiten
    unsigned int iCodeAnzahl = 0, iCodeLaenge = 0;

    //Abfrage der Anzahl und Laenge der UEDE mit sinnvollen Obergrenzen
    do
    {
      cout << "Wie viele UEDE moechten sie vergleichen (min:2; max:5000) ?\t";
      cin >> iCodeAnzahl;

      cout << endl << "Welche Laenge sollen die einzelnen UEDE haben (min:1; max:5000) ?\t";
      cin >> iCodeLaenge;

      if (iCodeAnzahl < 2 || iCodeAnzahl > 5000 || iCodeLaenge < 1 || iCodeLaenge > 5000)

          cout << endl << "Ungueltige Eingabe!" << endl << endl;

    }
    while (iCodeAnzahl < 2 || iCodeAnzahl > 5000 || iCodeLaenge < 1 || iCodeLaenge > 5000);


    iAnzahlSpeicherworte = laengeDerUEDE(iCodeLaenge); // Anzahl der Shorts fuer eine UEDE
    generiereUEDEs(iCodeAnzahl, iCodeLaenge);


    char aktion=0;
    vector<Vektorelement> uede;

    do
    {
        cout << endl << "Was moechten sie tun?" << endl
             << "1. Alle Codewoerter ausgeben" << endl
             << "2. Die minimale Distanz-Distanz ausgeben" << endl
             << endl
             << "4. Programmende" << endl;

        cin >> aktion;

        switch(aktion)
        {
        case '1':

            for (unsigned int i=0; i < iCodeAnzahl; ++i)
            {
                uede = getUEDE(i);

                ausgabeUEDE(uede);
            }

            break;

        case '2':

            minimaleDistanz(iCodeAnzahl);

            break;

        case '4':

            break;

        default:

            cout << "Nur 1,2,4 zugelassen";
        }
    }
    while (aktion!='4');

    return 0;
}
