/*
Dieses Programm ermittelt die Teilerreste zu allen 16 4-Bit-Kombinationen.
(Alle Polynome dritten Grades)
Ablauf:
1.) Bit-Kombination wird erstellt (Dezimalaequivalent) z.B. 7 = 0111
2.) Multiplikation mit x**4 -> Bitshift um 4 nach links -> 01110000
3.) Ueberpruefung, ob die ersten vier Bits auf Null gesetzt sind. Im Beispiel ist die erste Stelle auf Null gesetzt
4.) Dividend wird in tempordre Variable geladen (Dividend = 00011101)
5.) Der Dividend wird um Anzahl der nicht gesetzten Stellen von links nach links verschoben -> 00111010
6.) Die Bitkombination und der Dividend werden XOR verknuepft, so dass die gleichen
Stellen wegfallen (Siehe manuelle Polynomdivision mit Rest).
7.) Die Schritte 3.) bis 6.) werden so lange wiederholt, bis die ersten 4 Stellen
der Bitkombination Null sind, da dann keine weitere Polynomdivision durchgefuehrt
werden kann. Die letzten 4 Stellen geben dann den Teilerrest und werden im Programm und
ueber einen Stream in eine Textdatei ausgegeben.
 */


#include <iostream> // Zur Ausgabe von Daten
#include <fstream> // Dateistreams

using namespace std;

// Es wird unsigned char als Datentyp gewdhlt
// da er 8 Bit zur Verfuegung stellt und in dieser Aufgabe nicht mehr benoetigt wird

// speichert die aktuelle Bitfolge
unsigned char polynom = 0;

const unsigned char dividend = 29; // Entspricht dem Binaerequivalent 00011101, also dem vorgegebenen Polynom!

//unsigned char teilerrest = 0; // Speichert den Teilerrest der Polynomdivision

unsigned char generatorPolynom = 0; // Zwischenspeicher fuer den Dividend der Polynomdivision

unsigned char linksshift = 0; // Wert, um den der Dividend per shift nach links geschoben werden muss

// Stream fuer die Codedatei oeffnen
ofstream codetabelle("Versuch_09_Teilaufgabe_2_Tabelle.txt");


/*
 * Polynomdivision mit Rest
 * Rueckgabewert: Rest
 */
unsigned char teilerrest(unsigned char polynom)
{
	for (linksshift=7; linksshift > 3; --linksshift)
		if (1 & (polynom >> linksshift))
			polynom ^= dividend << (linksshift-4);

	return polynom & 0x0F;
}


int main(void)
{
	// Die Zahlen 0 bis 15 entsprechen den Bitfolgen der Laenge vier
	for (int dezimalaequivalent = 0; dezimalaequivalent <= 15; dezimalaequivalent++)
	{
		// Setzt das Dezimaldquivalent als aktuelles Polynom
		polynom = dezimalaequivalent;
		polynom <<= 4; // Linksshift, Multiplikation mit x**4

		// Typecast zum unsigned short, um die Lesbarkeit zu verbessern, da die ersten 16 Zeichen des ASCII-Codes nicht alle
		// lesbar sind
		unsigned short iTeilerrest = teilerrest(polynom);

		codetabelle << dezimalaequivalent << " " << iTeilerrest << endl;

		cout << dezimalaequivalent << " " << iTeilerrest << endl;
	}

	codetabelle.close(); // Stream auf die Datei schlie_en
	//  system("PAUSE");

	return 0;
}

